UNIT mChatClientVerbindung;
{**************************************************************************
  mChatClientVerbindung.pas

  Die wichtigste Klasse fr den ChatClient.

  Um dem MVC-Prinzip (Trennung von "Intelligenz" und "Oberflche") gerecht
  zu werden, verwende ich hier eine CallBack-Funktion, d.h die Oberflche
  bergibt bei Erzeugung einer ChatClientVerbindung eine Methode (als
  Methodentyp!) fr die Ausgabe.
  Da Methodentypen u.U. noch nicht bekannt sind, ist ansonsten auch die
  bergabe eines Zeilenbereichs o.. mglich (verstt aber gegen das MVC-
  Prinzip).

  Diese Datei gehrt zu den Materialien zu dem Workshop
  "Zentralabitur: Kommunikation in Netzwerken" des 5. Informatiktag
  in Paderborn.
  Dieses Projekt ist im Frhjahr 2006 im Unterricht meines GK Informatik
  Jgst. 12 entwickelt und fr den Workshop nur leicht berarbeitet worden.
  Sie darf fr eigene Unterrichtszwecke frei verwendet werden.
  Eine anderweitige Verffentlichung ist nur mit meiner Erlaubnis zulssig.

  (c) Joachim Deckers, <joachim@deckers.name>, 03.04.2006
***************************************************************************}

interface

//--------------------  ggf Uses-Liste anpassen !  --------------------
uses mClientVerbindung, mSuM, mZeilenbereich, mTextwerkzeug, mTextfeld;

const ZEILENENDE=#13#10;

type

  Ausgabemethode = procedure (const pMeldung: Zeichenkette) of object ;

  ChatClientVerbindung = class(ClientVerbindung)
  private //Attribute
    hatTextwerkzeug : Textwerkzeug;
    zUsername: Zeichenkette;
    zStatus : GanzeZahl;

    ausgabe: Ausgabemethode;

  public //Methoden
    constructor init (pServer: Zeichenkette; pPortNr: GanzeZahl; pZb: TZeilenbereich; pUser: Zeichenkette; pAusgabe: Ausgabemethode); // nicht berschreibbar
    procedure bearbeiteNachricht (pNachricht: Zeichenkette); override;
    procedure sendeOeffentlich (pMsg: Zeichenkette);
    procedure liste;
    procedure meldeAb;
    procedure sendePrivat (pPMsg : Zeichenkette; pEmpfaenger: Zeichenkette);
    function  fehler: Wahrheitswert;
    destructor gibFrei;
   end;

implementation

//+---------------------------------------------------------------------
//|         ChatClientVerbindung: Methodendefinition
//+---------------------------------------------------------------------

//-------- init (public) -----------------------------------------------
constructor ChatClientVerbindung.init (pServer: Zeichenkette; pPortNr: GanzeZahl;
             pZb: TZeilenbereich; pUser: Zeichenkette; pAusgabe: Ausgabemethode);
begin
  ausgabe         := pAusgabe;
  zUsername       := pUser;
  hatTextwerkzeug := Textwerkzeug.init;
  zStatus         := 0;
  inherited init(pServer, pPortNr, FALSCH);
end;

//-------- bearbeiteNachricht (public) ---------------------------------
procedure ChatClientVerbindung.bearbeiteNachricht (pNachricht: Zeichenkette);
var lIndex,
    lRaute,
    lRaute2    : GanzeZahl;
    lNeu,lNeu2 : Zeichenkette;

begin
  // Zeilenendezeichen lschen
  lIndex := hatTextwerkzeug.positionVon(pNachricht,#13);
  if lIndex>0 then
    pNachricht := hatTextwerkzeug.teilZeichenkette(pNachricht,1,lIndex-1);

  if pNachricht <> '' then
     case zStatus of
     0 : begin
           if pNachricht[1] = '+' then
             begin
               self.sende('USER ' + zUsername+ZEILENENDE);
               zStatus := 1;
             end
           else if pNachricht[1]= '-' then
             begin
               ausgabe('Fehler: '+hatTextwerkzeug.teilZeichenkette(pNachricht, 2,hatTextwerkzeug.laenge(pNachricht)));
               zStatus := 3;
             end;
         end;

     1 :  if pNachricht[1] = '+' then
              zStatus := 2
          else if pNachricht[1]= '-' then
             begin
               ausgabe('Fehler: '+hatTextwerkzeug.teilZeichenkette(pNachricht, 2,hatTextwerkzeug.laenge(pNachricht)));
               zStatus := 3;
             end;
     2 :  case pNachricht[1] of
            '*' : begin
                    lRaute := hatTextwerkzeug.positionVon(pNachricht,'#');
                    ausgabe('Von ' + hatTextwerkzeug.teilZeichenkette(pNachricht, 2,lRaute-1) + ': ' + hatTextwerkzeug.teilZeichenkette(pNachricht, lRaute+1 ,hatTextwerkzeug.laenge(pNachricht)));
                  end;

            '~' : begin
                    lRaute := hatTextwerkzeug.positionVon(pNachricht,'#');
                    lNeu   := (hatTextwerkzeug.Textohne(pNachricht, 1, lRaute)); //  Nachricht ohne '~ABSENDER#'
                    lRaute2:= hatTextwerkzeug.positionVon(lNeu, '#');
                    ausgabe('Private Nachricht von '
                      + hatTextwerkzeug.teilZeichenkette(pNachricht, 2,lRaute-1)
                      + ' an ' + hatTextwerkzeug.teilZeichenkette(lNeu, 1 , lRaute2 - 1)
                      + ': ' + hatTextwerkzeug.teilZeichenkette(lNeu, lRaute2+1 ,hatTextwerkzeug.laenge(lNeu)));
                  end;

            'S' : ausgabe('Servernachricht: ' + hatTextwerkzeug.teilZeichenkette(pnachricht, 2,hatTextwerkzeug.laenge(pNachricht)));

            '+' : if pNachricht = '+Tschss' then
                    begin
                      ausgabe('Verbindung getrennt');
                      self.gibFrei; // gefhrlich!
                    end;

           '-' :  ausgabe('Fehler: '+hatTextwerkzeug.teilZeichenkette(pNachricht,2,hatTextwerkzeug.laenge(pNachricht)));

           '$' :  begin
                     pNachricht := hatTextwerkzeug.teilZeichenkette(pNachricht,2,hatTextwerkzeug.laenge(pNachricht));
                     for lIndex:=1 to hatTextwerkzeug.laenge(pNachricht) do
                       if pNachricht[lIndex]='#' then pNachricht[lIndex]:=',';
                     ausgabe('Angemeldete User: '+pNachricht);  
                   end;
          end;

     3 :  begin
            // Fehlerstatus - wird nicht mehr verlassen => das ist nicht optimal!
            ausgabe(hatTextwerkzeug.teilZeichenkette(pNachricht, 2,hatTextwerkzeug.laenge(pNachricht)));
          end;
  end;
end;


procedure ChatClientVerbindung.sendeOeffentlich(pMsg: Zeichenkette);
begin
  self.sende('MSG ' + pMsg+ZEILENENDE);
end;

procedure ChatClientVerbindung.liste;
begin
  self.sende('LIST'+ZEILENENDE);
end;

procedure ChatClientVerbindung.sendePrivat(pPMsg: Zeichenkette; pEmpfaenger: Zeichenkette);
begin
  self.sende('PMSG ' + pEmpfaenger + '#' + pPMsg+ZEILENENDE);
end;

destructor ChatClientVerbindung.gibFrei;
begin
  if zStatus > 0 then
     self.sende('BYE'+ZEILENENDE);
  inherited gibFrei;
end;

procedure ChatClientVerbindung.meldeAb;
begin
  self.sende('BYE'+ZEILENENDE);
end;

function ChatClientVerbindung.fehler: Wahrheitswert;
begin
  result := zStatus = 3;
end;

end.

