{*******************************************************************************
 *                            Zeichenbereich Version 1.1                       *
 *                (c) 2002 Ulrich Borghoff    Gymnasium Wilnsdorf              *
 *                           eMail: sum-delphi@web.de                          *
 * *****************************************************************************
 * Diese Komponenten Version drfte unter Delphi ab Version ab 4 laufen        *
 *******************************************************************************
 * Bitte ndern Sie diese Datei nicht ab, sondern teilen den Entwicklern Ihre  *
 * Verbesserungsvorschlge oder Fehler (und ggf. deren Berichtigung) mit.      *
 *                                                                             *
 * Danke fr Ihre Mitarbeit.                                                   *
 *******************************************************************************}


unit mZeichenbereich;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, mSuM;

const NEUERABSATZ=#13#10;
      ZEILENUMBRUCH=#13#10;

type
  TZeichenbereich = class(TCustommemo)

 private
    FFokusErhalten, FFokusVerloren, FTextGeaendert, FEingabeBestaetigt: tNotifyEvent;

    procedure setzeZustand(pAktiv:Wahrheitswert);
    procedure setzeSichtbarkeit(pSichtbar:Wahrheitswert);
    procedure setzeText(pInhalt:Zeichenkette);

  protected
    procedure doEnter; override;
    procedure doExit; override;
    procedure change; override;
    procedure KeyPress(var Key: Char); override;

  public

    constructor Create(AOwner: TComponent); override;
    constructor init(pLinks,pOben, pBreite, pHoehe: Zahl);  overload;
    constructor init(pLinks,pOben, pBreite, pHoehe: Zahl; pInhalt:Zeichenkette);  overload;

    procedure setzePosition(pLinks, pOben : Zahl); virtual;
    procedure setzeGroesse(pBreite, pHoehe : Zahl); virtual;

    function links:Zahl;  virtual;
    function oben:Zahl; virtual;
    function breite:Zahl;  virtual;
    function hoehe:Zahl; virtual;

    procedure verstecke; virtual;
    procedure zeige; virtual;
    function IstSichtbar:Wahrheitswert; virtual;

    procedure aktiviere; virtual;
    procedure deaktiviere; virtual;
    function istAktiviert: Wahrheitswert; virtual;

    procedure setzeFokus; virtual;
    function besitztFokus: Wahrheitswert; virtual;

    procedure setzeInhalt(pInhalt:Zeichenkette); overload; virtual;
    procedure setzeInhalt(pInhalt:Zahl); overload; virtual;

    function InhaltIstZeichenkette:Wahrheitswert; virtual;
    function InhaltIstZahl:Wahrheitswert; virtual;
    function InhaltIstGanzeZahl:Wahrheitswert; virtual;

    function inhaltAlsText:Zeichenkette; virtual;

    function inhaltAlsZahl:Zahl; virtual;
    function inhaltAlsGanzeZahl:GanzeZahl; virtual;

    procedure setzeSchriftart(pArt:Zeichenkette);
    procedure setzeSchriftGroesse(pGroesse:GanzeZahl);
    procedure setzeSchriftFarbe(pFarbe:GanzeZahl);
    procedure setzeSchriftStil(pStil:GanzeZahl);

    function istMarkiert : Wahrheitswert;

    function anzahl : GanzeZahl;

    function markierungsAnfang:GanzeZahl;
    function markierungsEnde:GanzeZahl;
    function markierterInhalt:Zeichenkette;
    procedure setzeMarkierung(pAnfang, pEnde : GanzeZahl);
    procedure markiereAlles;
    procedure markiereNichts;

    procedure loescheMarkierung;
    procedure loesche(pAnfang, pEnde :GanzeZahl);
    procedure loescheAlles;

    function teilInhalt(pAnfang, pEnde : GanzeZahl):Zeichenkette;

    procedure fuegeEin(pText:Zeichenkette; pStelle:GanzeZahl);
    procedure haengeAn(pText:Zeichenkette);

    procedure gibFrei; virtual;

    published

    property _Inhalt: Zeichenkette read inhaltAlsText
                             write setzeText;
    property _IstAktiviert : Wahrheitswert read istAktiviert write SetzeZustand;
    property _IstSichtbar : Wahrheitswert read IstSichtbar write SetzeSichtbarkeit;
    property font;
    property FokusErhalten : tNotifyEvent read FFokusErhalten write FFokusErhalten;
    property FokusVerloren : tNotifyEvent read FFokusVerloren write FFokusVerloren;
    property TextGeaendert : tNotifyEvent read FTextGeaendert write FTextGeaendert;
  //  property EingabeBestaetigt : tNotifyEvent read FEingabeBestaetigt write FEingabeBestaetigt;

  end;

procedure Register;

implementation

constructor TZeichenbereich.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  self.setzeInhalt('');
  self.HideSelection:=false;
end;

constructor TZeichenbereich.init(pLinks,pOben, pBreite, pHoehe: Zahl);
begin
  inherited create(screen.activeForm);
  ControlStyle := [csSetCaption, csOpaque, csDoubleClicks];
  TabStop := True;
  self.setzePosition(pLinks,pOben);
  self.setzeGroesse(pBreite,pHoehe);
  self.setzeText('');
  self.parent:=screen.activeForm;
end;

constructor TZeichenbereich.init(pLinks,pOben, pBreite, pHoehe: Zahl; pInhalt:Zeichenkette);
begin
  self.init(pLinks,pOben, pBreite, pHoehe);
  self.setzeInhalt(pInhalt);
end;

procedure TZeichenbereich.setzePosition(pLinks, pOben:Zahl);
begin
 self.left:=round(pLinks);
 self.Top:=round(pOben);
end;

procedure TZeichenbereich.setzeGroesse(pBreite, pHoehe:Zahl);
begin
 self.Width:=round(pBreite);
 self.Height:=round(pHoehe);
end;

function TZeichenbereich.links:Zahl;
begin
 result:=self.left
end;

function TZeichenbereich.oben:Zahl;
begin
  result:=self.top;
end;

function TZeichenbereich.breite:Zahl;
begin
  result:=self.width;
end;

function TZeichenbereich.hoehe:Zahl;
begin
  result:=self.height;
end;

procedure TZeichenbereich.verstecke;
begin
 self.visible:=false;
end;

procedure TZeichenbereich.zeige;
begin
 self.visible:=true;
end;

function TZeichenbereich.IstSichtbar:Wahrheitswert;
begin
  result:=self.visible;
end;

procedure TZeichenbereich.aktiviere;
begin
 self.enabled:=true;
end;

procedure TZeichenbereich.deaktiviere;
begin
 self.enabled:=false;
end;

function TZeichenbereich.istAktiviert: Wahrheitswert;
begin
 result:=self.enabled;
end;

procedure TZeichenbereich.setzeFokus;
begin
 screen.ActiveForm.FocusControl(self);
 self.AutoSelect:=true;
 self.SelectAll;
end;

function TZeichenbereich.besitztFokus: Wahrheitswert;
begin
 result:=self.focused;
end;

procedure TZeichenbereich.setzeText(pInhalt:Zeichenkette);
begin
 self.text:=pInhalt;
end;

procedure TZeichenbereich.setzeInhalt(pInhalt:Zeichenkette);
begin
 self.text:=pInhalt;
 self.update;
end;

procedure TZeichenbereich.setzeInhalt(pInhalt:Zahl);
begin
  self.setzeText(floatToStr(pInhalt));
  self.update;
end;

function TZeichenbereich.InhaltIstZeichenkette:Wahrheitswert;
begin
  result:=true;
end;

function TZeichenbereich.InhaltIstZahl:Wahrheitswert;
  var lWert: Zahl;
      lcode : integer;
begin
  val(self.inhaltAlsText,lWert,lCode);
  result:=(lcode=0);
end;

function TZeichenbereich.InhaltIstGanzeZahl:Wahrheitswert;
  var lWert: GanzeZahl;
      code : integer;
begin
  val(self.inhaltAlsText,lWert,Code);
  result:=(code=0);
end;

function TZeichenbereich.InhaltAlsText:Zeichenkette;
begin
 result := self.Text;
end;

function TZeichenbereich.inhaltAlsZahl:Zahl;
begin
  if self.InhaltistZahl then result:=strTofloat(self.inhaltAlsText)
  else
  begin
    MessageDlg('Eingabe keine Zahl.',mtError,[mbOK],0);
    result:=0;
  end;
end;

function TZeichenbereich.inhaltAlsGanzeZahl:GanzeZahl;
begin
  if self.InhaltIstGanzeZahl then result:=strToInt(self.inhaltAlsText)
  else
  begin
    MessageDlg('Eingabe keine ganze Zahl.',mtError,[mbOK],0);
    result:=0;
  end;
end;

procedure TZeichenbereich.setzeSchriftart(pArt:Zeichenkette);
begin
  self.Font.Name:=pArt;
end;

procedure TZeichenbereich.setzeSchriftGroesse(pGroesse:GanzeZahl);
begin
  self.Font.size:=pGroesse;
end;

procedure TZeichenbereich.setzeSchriftFarbe(pFarbe:GanzeZahl);
begin
  self.Font.Color:=SuMfarbumrechnung(pFarbe); {farbumrechnung aus mSuM}
end;

procedure TZeichenbereich.setzeSchriftStil(pStil:GanzeZahl);
begin
  self.Font.Style:=SuMstilumrechnung(pStil);  {stilumrechnung aus mSuM}
end;


procedure TZeichenbereich.gibFrei;
begin
 self.destroy;
end;

function TZeichenbereich.anzahl : GanzeZahl;
begin
  result:=length(self.inhaltAlsText);
end;

function TZeichenbereich.istMarkiert : Wahrheitswert;
begin
  result:=self.SelLength>0;
end;

function TZeichenbereich.markierungsAnfang:GanzeZahl;
begin
  if self.istMarkiert then result:=self.SelStart+1
  else result:=0;
end;

function TZeichenbereich.markierungsEnde:GanzeZahl;
begin
  if self.istMarkiert then result:=self.SelStart+self.SelLength
  else result:=0;
end;

function TZeichenbereich.markierterInhalt:Zeichenkette;
begin
  result:=self.SelText;
end;

procedure TZeichenbereich.setzeMarkierung(pAnfang, pEnde : GanzeZahl);
begin
  if (pAnfang<=pEnde) and (pEnde>0) then
  begin
    SelStart:=pAnfang-1;
    SelLength:=pEnde-pAnfang+1;
  end
  else self.markiereNichts;
end;

procedure TZeichenbereich.markiereAlles;
begin
  self.SelectAll;
end;

procedure TZeichenbereich.markiereNichts;
begin
  self.SelLength:=0;
end;


procedure TZeichenbereich.loescheMarkierung;
begin
  self.ClearSelection;
end;

procedure TZeichenbereich.loesche(pAnfang, pEnde :GanzeZahl);
begin
  self.setzeMarkierung(pAnfang, pEnde);
  self.loescheMarkierung;
end;

procedure TZeichenbereich.loescheAlles;
begin
  self.Clear;
end;


function TZeichenbereich.teilInhalt(pAnfang, pEnde : GanzeZahl):Zeichenkette;
begin
  result:=copy(self.text,pAnfang,pEnde-pAnfang+1);
end;


procedure TZeichenbereich.fuegeEin(pText:Zeichenkette; pStelle:GanzeZahl);
  var lText:zeichenkette;
begin
  lText:=self.Text;
  insert(pText,lText,pStelle);
  self.text:=lText;
end;

procedure TZeichenbereich.haengeAn(pText:Zeichenkette);
begin
  self.text:=self.text+pText;
end;

// private

procedure TZeichenbereich.setzeZustand(pAktiv:Wahrheitswert);
begin
 self.enabled:=pAktiv;
end;

procedure TZeichenbereich.SetzeSichtbarkeit(pSichtbar:Wahrheitswert);
begin
 self.visible:=pSichtbar;
end;


// Ereignisse

procedure TZeichenbereich.doEnter;
begin
  if assigned(FokusErhalten) then FokusErhalten(self);
  inherited doEnter;
end;

procedure TZeichenbereich.doExit;
begin
  if assigned(FokusVerloren) then FokusVerloren(self);
  inherited doExit;
end;

procedure TZeichenbereich.change;
begin
  if assigned(TextGeaendert) then TextGeaendert(self);
  inherited change;
end;

procedure TZeichenbereich.KeyPress(var Key: Char);
begin
 // if assigned(EingabeBestaetigt) and (key=#13) then EingabeBestaetigt(self);
  inherited KeyPress(key);
end;

// Registrierung

procedure Register;
begin
  RegisterComponents('Schule', [TZeichenbereich]);
end;


end.
